/* $Id: main.c,v 1.3 2010/06/27 22:19:15 simimeie Exp $
 * Main file for the HaWo moodlight.
 * This is the main file that glues it all together. It also contains all
 * functionality that is too small to require an extra file.
 * (C) Michael "Fox" Meier 2010
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/power.h>
#include <avr/sleep.h>
#include <stdio.h>
#include <string.h>
#include <util/delay.h>

#include "console.h"
#include "ledpwm.h"
#include "ircontrol.h"
#include "timers.h"
/* #include "eepromdata.h" */

uint8_t mcusr_mirror __attribute__ ((section (".noinit")));

/* This is needed to recover from a watchdog reset, as the watchdog
 * stays active after the reset.
 * The variable is just to make the reason of the last reset accessible
 * later. */
void get_mcusr(void) __attribute__((naked)) __attribute__((section(".init3")));
void get_mcusr(void) {
  mcusr_mirror = MCUSR;
  MCUSR = 0;
  wdt_disable();
}

int main(void)
{
  uint8_t i = 0;
  
  /* Load settings from EEPROM */
#if 0
  something = eeprom_read_byte(&ee_something);
#endif /* LEDMODULE */

  /* Initialize stuff */
  console_init();
  timers_init();
  ledpwm_init();
  ircontrol_init();
  
  wdt_enable(WDTO_2S);

  /* Prepare sleep mode */
  //set_sleep_mode(SLEEP_MODE_IDLE);
  //sleep_enable();

  /* All set up, enable interrupts and go. */
  sei();
  
  if (mcusr_mirror & _BV(WDRF)) {
    console_printpgm_P(PSTR("NOTE: last reset was from Watchdog Timer."));
  }
  if (mcusr_mirror & _BV(BORF)) {
    console_printpgm_P(PSTR("NOTE: last reset was from Brownout."));
  }
  
  while (1) { /* We should never exit */
    wdt_reset();
    /* i++;
    console_printhex8(i);
    ledpwm_setled(LEDPWM_REDLED, i);
    ledpwm_setled(LEDPWM_GREENLED, i+128);
    ledpwm_setled(LEDPWM_BLUELED, 255-i);
    _delay_ms(500); */
  }
}
